@@ -2,6 +2,7 @@ pub mod mem;
 pub mod rtne;
 pub mod arch;
 pub mod exec;
+pub mod ir;
 
 pub use mem::*;
 pub use rtne::*;
@@ -204,4 +205,136 @@ mod tests {
 
         assert_eq!(result, None)
     }
+
+    #[test]
+    fn test_tok() {
+        let tok = ir::Token::new(ir::INT_TOKEN, String::from("10"));
+
+        assert_eq!(tok.token_type, ir::INT_TOKEN);
+        assert_eq!(tok.token_value, String::from("10"));
+    }
+
+    #[test]
+    fn test_pos() {
+        let mut pos = match ir::Pos::from(String::from("1234\n5678\n90")) {
+            Ok(ok) => ok,
+            Err(err) => panic!("{}", err)
+        };
+
+        assert_eq!(pos.chr, '1');
+        assert_eq!(pos.pos, 0);
+        assert_eq!(pos.lno, 0);
+        assert_eq!(pos.onl, 0);
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        assert_eq!(pos.chr, '2');
+        assert_eq!(pos.pos, 1);
+        assert_eq!(pos.lno, 0);
+        assert_eq!(pos.onl, 1);
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        assert_eq!(pos.chr, '5');
+        assert_eq!(pos.pos, 5);
+        assert_eq!(pos.lno, 1);
+        assert_eq!(pos.onl, 1);
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        assert_eq!(pos.chr, '9');
+        assert_eq!(pos.pos, 10);
+        assert_eq!(pos.lno, 2);
+        assert_eq!(pos.onl, 1);
+
+        assert!(pos.has_next());
+
+        match pos.next() {
+            Some(some) => panic!("{}", some),
+            None => assert!(true)
+        };
+
+        assert_eq!(pos.has_next(), true);
+    }
+
+    #[test]
+    fn test_lexer() {
+        let mut lexer = match ir::Lexer::from(String::from("qwe rty uop")) {
+            Ok(ok) => ok,
+            Err(err) => panic!("{}", err)
+        };
+
+        let lex_res = match lexer.lex() {
+            Ok(ok) => ok,
+            Err(err) => panic!("{}", err)
+        };
+
+        let expected = vec![
+            ir::Token::new(ir::ID_TOKEN, String::from("qwe")),
+            ir::Token::new(ir::ID_TOKEN, String::from("rty")),
+            ir::Token::new(ir::ID_TOKEN, String::from("uop"))
+        ];
+
+        assert_eq!(lex_res.len(), expected.len());
+
+        let mut idx: usize = 0;
+
+        for e in lex_res {
+            assert_eq!(e, match expected.get(idx) {
+                Some(some) => some.clone(),
+                None => ir::Token::new(ir::INT_TOKEN, String::from("1"))
+            });
+            
+            idx = idx + 1;
+        }
+    }
+
+    #[test]
+    fn test_delex() {
+        assert!(true);
+    }
 }
